==> fap-0.8.1.tar.bz2: fap-0.8.1/src/audio/input/factory.h <==
/*
 *  Copyright (C) 2007-2008 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef FACTORY_H
#define FACTORY_H

#include "input.h"

#include <fx.h>
#include <new>
#include <FXArray.h>

#define IFACTORY	factory::instance()

class factory {

typedef input* (*inputFactory)();
typedef bool (*supportsFunc)(const char*);

typedef struct {
	const char* name;
	supportsFunc supports;
	inputFactory factory;
} inputs_t;

private:
	template <class backend>
	void regBackend();

private:
	FXArray<inputs_t>	_inputs;

protected:
	factory();
	factory(const factory&);
	factory& operator= (const factory&);
	~factory() {};

public:
	/// Get singleton instance
	static factory* instance();

	/// Get number of registered backends
	const int num() const { return _inputs.no(); }

	/// Get backend's name
	const char* name(int i) const { return _inputs[i].name; }

	/// Check if backend supports file
	const bool supports(int i, const char* file) const { return _inputs[i].supports(file); }

	/// Instantiate backend
	input* get(int i);
};

#endif

==> fap_0.8.8.orig.tar.bz2: fap-0.8.8/src/audio/input/factory.h <==
/*
 *  Copyright (C) 2007-2011 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef FACTORY_H
#define FACTORY_H

#include "input.h"

#include <fx.h>
#include <new>
#include <FXArray.h>

#define IFACTORY	factory::instance()

class factory {

typedef input* (*inputFactory)();
typedef bool (*supportsFunc)(const char*);

typedef struct {
	const char* name;
	supportsFunc supports;
	inputFactory factory;
} inputs_t;

private:
	template <class backend>
	void regBackend();

private:
	FXArray<inputs_t>	_inputs;

protected:
	factory();
	factory(const factory&);
	factory& operator= (const factory&);
	~factory() {};

public:
	/// Get singleton instance
	static factory* instance();

	/// Get number of registered backends
	const int num() const { return _inputs.no(); }

	/// Get backend's name
	const char* name(const int i) const { return _inputs[i].name; }

	/// Check if backend supports file
	const bool supports(const int i, const char* file) const { return _inputs[i].supports(file); }

	/// Instantiate backend
	input* get(const int i);
};

#endif
